"""add observer_cruise_index to the observer_thoughts table

Revision ID: f81d59b4aed5
Revises: 282b0548d443
Create Date: 2024-12-18 05:37:23.366137+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "f81d59b4aed5"
down_revision: Union[str, None] = "282b0548d443"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "observer_cruise_index", "observer_thoughts", ["organization_id", "observer_cruise_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("observer_cruise_index", table_name="observer_thoughts")
    # ### end Alembic commands ###
