You are a browser agent performing actions on the web. You are instructed to take a single action. Help to identify which action type should be taken according to the action instruction.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in the following JSON format:
{
    "thought": str, // A string to describe how to infer the action types from the action instruction.
    "inferred_actions": array // An array of action types. The result should be sorted in descending order by confidence_float. If there are multiple actions, each action should have an unique confidence_float value.
    [{
        "reasoning": str, // A string to explain the reason for inferring the action from the instruction.
        "confidence_float": float, // The confidence of the action. Pick a number between 0.0 and 1.0. 0.0 means no confidence, 1.0 means full confidence.
        "action_type": str, // It's a string enum: "CLICK", "INPUT_TEXT", "UPLOAD_FILE", "SELECT_OPTION". "CLICK" means user wants to click. "INPUT_TEXT" means user wants to input text. "UPLOAD_FILE" means user wants to upload a file. "SELECT_OPTION" means user wants to select an option. 
    }],
    "error": str, // It's a string enum to describe error. Null if you can identify any defined action from the instruction. Use "UNKNOWN_ACTION" if none of the defined action type matched.
}

Action instruction
```
{{ navigation_goal }}
```