The user is trying to achieve a goal in the browser assisted by a browser AI assistant.

According to the AI assistant's feedback, including reasoning and its message, there's no immediate action the assistant can take in the website.

Help the user decide what to do next based on the assistant's message. Here's the list of available actions:
- solve_captcha: the task is blocked by captcha and the assistant is asking the user to solve the captcha
- complete: the user goal has been achieved
- get_verification_code: the assistant is asking the user to provide a verification code (2FA, MFA or TOTP code). At this point, the code should have been sent to the user. If code hasn't been sent, do not return get_verification_code action.
- other: the assistant is asking the user to do something else

Return the action to take next in the following JSON format:
{
    "action": str // complete, solve_captcha, get_verification_code, other
    "useful_information": str // If there is any useful information the assistant has provided that contributes to the user goal, put it here.
}

User goal: {{ navigation_goal }}

Assistant reasoning: {{ assistant_reasoning }}

Assistant message: {{ assistant_message }}