"""Add suggested_title to task_generations table

Revision ID: 6de11b2be7c8
Revises: 8f237f00faeb
Create Date: 2024-08-23 20:12:13.426060+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "6de11b2be7c8"
down_revision: Union[str, None] = "8f237f00faeb"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("task_generations", sa.Column("suggested_title", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_generations", "suggested_title")
    # ### end Alembic commands ###
