You are given text that has been extracted from a file. Your task is to extract information from this text.

Extract information from the text and output it in the specified JSON schema format: {{ json_schema }}

Add as much details as possible to the output JSON object while conforming to the output JSON schema.

Do not ever include anything other than the JSON object in your output, and do not ever include any additional fields in the JSON object.

If you are unable to extract the requested information for a specific field in the json schema, please output a null value for that field.

You are given the following text

{{ extracted_text_content }}
