"""Add workflow_permanent_id and version to workflows table

Revision ID: bf561125112f
Revises: 8792454ce498
Create Date: 2024-05-14 01:14:15.024575+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "bf561125112f"
down_revision: Union[str, None] = "8792454ce498"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflows", sa.Column("workflow_permanent_id", sa.String(), nullable=True))
    op.add_column("workflows", sa.Column("version", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "version")
    op.drop_column("workflows", "workflow_permanent_id")
    # ### end Alembic commands ###
