"""Add failure_reason column to workflow_runs

Revision ID: 1909715536dc
Revises: b8f9e09e181d
Create Date: 2024-11-15 02:51:33.553177+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1909715536dc"
down_revision: Union[str, None] = "b8f9e09e181d"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("failure_reason", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "failure_reason")
    # ### end Alembic commands ###
