"""Add IN and JP proxylocation

Revision ID: b8f9e09e181d
Revises: 2873c5c8c41e
Create Date: 2024-11-04 19:14:36.603689+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "b8f9e09e181d"
down_revision: Union[str, None] = "2873c5c8c41e"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute("ALTER TYPE proxylocation ADD VALUE 'RESIDENTIAL_IN'")
    op.execute("ALTER TYPE proxylocation ADD VALUE 'RESIDENTIAL_JP'")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
