"""add description to workflow run block

Revision ID: 32e2f138f7fd
Revises: 521241e64aed
Create Date: 2025-01-03 23:49:40.290858+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "32e2f138f7fd"
down_revision: Union[str, None] = "521241e64aed"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_run_blocks", sa.Column("description", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_run_blocks", "description")
    # ### end Alembic commands ###
